#!/usr/bin/perl -w 
############################## check_snmp_ups ###################
# Version : 2.2.0
# Date : October 20 2009
# Author  : Juergen Vigna <juergen.vigna@wuerth-phoenix.com>
#
# Licence : GPL - http://www.fsf.org/licenses/gpl.txt
# Changelog : 
# 2.3: Include of APC UPS MIB base (Patrick Zambelli )
# www.neteye-blog.it
#
#################################################################
#
# Help : ./check_snmp_ups.pl -h
#

use strict;
use Net::SNMP;
use Getopt::Long;

# Nagios specific

use lib "/usr/lib/nagios/plugins";
use utils qw(%ERRORS $TIMEOUT);
#my $TIMEOUT = 15;
#my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);


my @Nagios_state = ("UNKNOWN","OK","WARNING","CRITICAL"); # Nagios states coding

# SNMP Datas

# Net Vision UPS MIB's
my $netvision_base_MIB	=	"1.3.6.1.4.1.4555"; # NetVision UPS base table
my $netvision_alert_table =	"1.3.6.1.4.1.4555.1.1.1.1.6.3";
my @netvision_alert_leaf = (    # 82 elements 0-81 id's are array-index + 1
 'AlarmBatteryBad',
 'AlarmOnBattery',
 'AlarmLowBattery',
 'AlarmDepletedBattery',
 'AlarmTempBad',
 'AlarmInputBad',
 'AlarmOutputBad',
 'AlarmOutputOverload',
 'AlarmOnBypass',
 'AlarmBypassBad',
 'AlarmOutputOffAsRequested',
 'AlarmUpsOffAsRequested',
 'AlarmChargerFailed',
 'AlarmUpsOutputOff',
 'AlarmUpsSystemOff',
 'AlarmFanFailure',
 'AlarmFuseFailure',
 'AlarmGeneralFault',
 'AlarmDiagnosticTestFailed',
 'AlarmCommunicationLost',
 'AlarmAwaitingPower',
 'AlarmShutdownPending',
 'AlarmShutdownImminent',
 'AlarmTestInProgress',
 'AlarmPowerSupplyFault',
 'AlarmAuxMainFail',
 'AlarmManualBypassClose',
 'AlarmShortCircuit',
 'AlarmBatteryChargerFailure',
 'AlarmInverterOverCurrent',
 'AlarmInverterDistorsion',
 'AlarmPrechargeVoltageFail',
 'AlarmBoostTooLow',
 'AlarmBoostTooHigh',
 'AlarmBatteryTooHigh',
 'AlarmImproperCondition',
 'AlarmOverloadTimeout',
 'AlarmControlSystemFailure',
 'AlarmDataCorrupted',
 'AlarmPllFault',
 'AlarmInputGeneralAlarm',
 'AlarmRectifierGeneralAlarm',
 'AlarmBoostGeneralAlarm',
 'AlarmInverterGeneralAlarm',
 'AlarmBatteryGeneralAlarm',
 'AlarmOutputOver',
 'AlarmOutputUnder',
 'AlarmBypassGeneralAlarm',
 'AlarmStopForOverload',
 'AlarmImminentStop',
 'AlarmModule1Alarm',
 'AlarmModule2Alarm',
 'AlarmModule3Alarm',
 'AlarmModule4Alarm',
 'AlarmModule5Alarm',
 'AlarmModule6Alarm',
 'AlarmExternalAlarm1',
 'AlarmExternalAlarm2',
 'AlarmExternalAlarm3',
 'AlarmExternalAlarm4',
 'AlarmEService',
 'AlarmRedundancyLost',
 'AlarmPeriodicServiceCheck',
 'AlarmAllTransferDisabled',
 'AlarmAutoTransferDisabled',
 'AlarmBatteryRoom',
 'AlarmManualBypass',
 'AlarmBatteryDischarged',
 'AlarmInsufficientResources',
 'AlarmOptionalBoards',
 'AlarmRectifierFault',
 'AlarmBoostFault',
 'AlarmInverterFault',
 'AlarmParallelModuleFault',
 'AlarmGenSetGeneral',
 'AlarmGenSetFault',
 'AlarmEmergencyStopActive',
 'AlarmBatteryCircuitOpen',
 'AlarmFansFailure',
 'AlarmPhaseRotationFault',
 'AlarmA62',
 'AlarmA63'
);
my $netvision_capacity      = "1.3.6.1.4.1.4555.1.1.1.1.2.4.0";
my $netvision_load_table    = "1.3.6.1.4.1.4555.1.1.1.1.4.4.1.4";
my $netvision_load1         = "1.3.6.1.4.1.4555.1.1.1.1.4.4.1.4.1";
my $netvision_load2         = "1.3.6.1.4.1.4555.1.1.1.1.4.4.1.4.2";
my $netvision_load3         = "1.3.6.1.4.1.4555.1.1.1.1.4.4.1.4.3";
my $netvision_voltage_table = "1.3.6.1.4.1.4555.1.1.1.1.3.3.1.2"; # voltage value * 10
my $netvision_voltage1      = "1.3.6.1.4.1.4555.1.1.1.1.3.3.1.2.1";
my $netvision_voltage2      = "1.3.6.1.4.1.4555.1.1.1.1.3.3.1.2.2";
my $netvision_voltage3      = "1.3.6.1.4.1.4555.1.1.1.1.3.3.1.2.3";

# unknown(1),batteryNormal(2),batteryDischarging(3),batteryDischarged(4),batteryFailure(5)
my $netvision_energy_batstatus    = "1.3.6.1.4.1.4555.1.1.2.1.2.1.0";
# in percent
my $netvision_energy_batcapacity  = "1.3.6.1.4.1.4555.1.1.2.1.2.4.0";
my $netvision_energy_inpvoltage   = "1.3.6.1.4.1.4555.1.1.2.1.3.2.1.2";
# other(1),none(2),normal(3),battery(4)
my $netvision_energy_outstatus    = "1.3.6.1.4.1.4555.1.1.2.1.4.1.0";
# in 0.1 volt
my $netvision_energy_outvoltage   = "1.3.6.1.4.1.4555.1.1.2.1.4.2.0";

#
# General Battery Status check
#
my $general_battery_status = "1.3.6.1.2.1.33.1.2.1.0";
my $general_output_source  = "1.3.6.1.2.1.33.1.4.1.0";
my %upsBatteryStatuses = (
    1 => "unknown(1)",
    2 => "batteryNormal(2)",
    3 => "batteryLow(3)",
    4 => "batteryDepleted(4)"
);

my %upsOutputSources = (
    1 => "other(1)",
    2 => "none(2)",
    3 => "normal(3)",
    4 => "bypass(4)",
    5 => "battery(5)",
    6 => "booster(6)",
    7 => "reducer(7)"
);
my $general_output_voltage_table = "1.3.6.1.2.1.33.1.4.4.1.2";
my $general_output_current_table = "1.3.6.1.2.1.33.1.4.4.1.3";
my $general_output_power_table   = "1.3.6.1.2.1.33.1.4.4.1.4";
my $general_output_load_table    = "1.3.6.1.2.1.33.1.4.4.1.5";
my $general_capacity             = "1.3.6.1.2.1.33.1.2.4.0";
my $general_input_voltage_table  = "1.3.6.1.2.1.33.1.3.3.1.3";
my $general_input_current_table  = "1.3.6.1.2.1.33.1.3.3.1.4";
my $general_input_power_table    = "1.3.6.1.2.1.33.1.3.3.1.5";


#
# APC UPS
#
my $apcups_base_MIB =    "1.3.6.1.4.1.318"; # APC ATS base table
my $apcups_battery_capacity = ".1.3.6.1.4.1.318.1.1.1.2.2.1.0";
my $apcups_battery_temp = ".1.3.6.1.4.1.318.1.1.1.2.2.2.0";
my $apcups_battery_voltage = ".1.3.6.1.4.1.318.1.1.1.2.2.8.0";
my $apcups_input_voltage = ".1.3.6.1.4.1.318.1.1.1.9.2.3.1.3.1.1";
my $apcups_output_load = ".1.3.6.1.4.1.318.1.1.1.9.3.3.1.10.1.1";

#
# APC Automatic Transfer Switch (no UPS but ...)
#
my $apcats_base_MIB =	 "1.3.6.1.4.1.318"; # APC ATS base table
my $apcats_input_table = ".1.3.6.1.4.1.318.1.1.8.5.3.3.1.3";
my $apcats_input_a =	 ".1.3.6.1.4.1.318.1.1.8.5.3.3.1.3.1.1.1";
my $apcats_input_b =	 ".1.3.6.1.4.1.318.1.1.8.5.3.3.1.3.2.1.1";

#
# Generex Generic UPS Module used by many UPS's
# Chloride Power
#
my $generex_alarms           = "1.3.6.1.2.1.33.1.6.1.0";
my $generex_battery_base     = "1.3.6.1.2.1.33.1.2";
my $generex_battery_status   = "1.3.6.1.2.1.33.1.2.1.0";
my $generex_battery_minact   = "1.3.6.1.2.1.33.1.2.2.0"; # Minutes On Battery
my $generex_battery_mintot   = "1.3.6.1.2.1.33.1.2.3.0"; # Minutes Remaining
my $generex_battery_capacity = "1.3.6.1.2.1.33.1.2.4.0"; # Charge Remaining
my $generex_battery_voltage  = "1.3.6.1.2.1.33.1.2.5.0";
my $generex_battery_current  = "1.3.6.1.2.1.33.1.2.6.0";
my $generex_battery_temp     = "1.3.6.1.2.1.33.1.2.7.0";
my $generex_output_source    = "1.3.6.1.2.1.33.1.4.1.0"; # 1: unknown 2: none 3: ok 4: bypass 5: battery 6: booster 7: reducer
my $generex_output_base      = "1.3.6.1.2.1.33.1.4.4.1";
my $generex_output_voltage   = "1.3.6.1.2.1.33.1.4.4.1.2"; # IDX for more indexes
my $generex_output_current   = "1.3.6.1.2.1.33.1.4.4.1.3"; # IDX for more indexes
my $generex_output_power     = "1.3.6.1.2.1.33.1.4.4.1.4"; # IDX for more indexes
my $generex_output_load      = "1.3.6.1.2.1.33.1.4.4.1.5"; # percent, IDX for more indexes
my $generex_input_base       = "1.3.6.1.2.1.33.1.3.3.1";
my $generex_input_voltage    = "1.3.6.1.2.1.33.1.3.3.1.3"; # IDX for more indexes
my $generex_input_current    = "1.3.6.1.2.1.33.1.3.3.1.4"; # IDX for more indexes
my $generex_input_power      = "1.3.6.1.2.1.33.1.3.3.1.5"; # IDX for more indexes

## OID LIST ##
my $mge_ups_model          = ".1.3.6.1.4.1.705.1.1.1.0";
my $mge_internal_temp      = ".1.3.6.1.4.1.705.1.5.7.0"; # degrees
my $mge_battery_percentage = ".1.3.6.1.4.1.705.1.5.2.0";
my $mge_battery_fault      = ".1.3.6.1.4.1.705.1.5.9.0"; # 1 yes, 2 no
my $mge_input_outage       = ".1.3.6.1.4.1.705.1.6.4.0"; # 1: ok, 2: voltage out tolerance, 3: freq out tolerance, 4: no voltage.
my $mge_output_overload    = ".1.3.6.1.4.1.705.1.7.2.1.4.1"; # %
my $mge_output_on_bypass   = ".1.3.6.1.4.1.705.1.7.3.0"; # 1: yes, 2: no
## NOT USED ##
my $mge_battery_voltage = ".1.3.6.1.4.1.705.1.5.5.0";                      # dV
my $mge_input_voltage = ".1.3.6.1.4.1.705.1.6.2.1.2.1";                    # dV
my $mge_output_voltage = ".1.3.6.1.4.1.705.1.7.2.1.2.1";                   # dV

my $delta_output_source    = "1.3.6.1.2.1.33.1.4.1.0"; # 1: unknown 2: none 3: ok 4: bypass 5: battery 6: booster 7: reducer
my $delta_output_load      = "1.3.6.1.2.1.33.1.4.4.1.5"; # percent, IDX for more indexes

# Globals

my $Version='1.0';

my $o_host = 	undef; 		# hostname
my $o_community = undef; 	# community
my $o_port = 	161; 		# port
my $o_help=	undef; 		# wan't some help ?
my $o_verb=	undef;		# verbose mode
my $o_version=	undef;		# print version
my $o_timeout=  undef; 		# Timeout (Default 5)
my $o_perf=     undef;          # Output performance data
my $o_version2= undef;          # use snmp v2c
# check type  
my $o_check_type= "general";	# default NetVision
my @valid_types	=("general","netvision","socomec","generex","chloride","apcats", "apcups","mge","nvenergy","delta");
my $o_battery=  undef;          # string for option (w,c)
my $o_load=     undef;          # string for option (w,c)
my $o_w_load=	75;		# max used load warning level
my $o_c_load=	90;		# max used load warning level
my $o_w_battery=80;     	# min battery remaining critical level
my $o_c_battery=50;	        # min battery remaining critical level
my $o_w_battery_temp=45;        # warning battery temperature
my $o_c_battery_temp=55;        # critical  battery temperature
my $o_voltage=  200;    	# max used voltage warnign level
my $o_output_lines=1;  	        # output lines used
my $o_input_lines=1;   	        # input lines used

# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;          # V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password

# functions

sub p_version { print "check_snmp_ups version : $Version\n"; }

sub print_usage {
    print "Usage: $0 [-v] -H <host> -C <snmp_community> [-2] | (-l login -x passwd [-X pass -L <authp>,<privp>])  [-p <port>] -T (general|netvision|socomec|generex|chloride|apcats|apcups|mge|nvenergy|delta) [-B <warn,crit>] [-O <warn,crit>] [-A <voltage>] [-f] [-t <timeout>] [-o <lines>] [-i <lines>] [-V]\n";
}

sub isnnum { # Return true if arg is not a number
  my $num = shift;
  if ( $num =~ /^(\d+\.?\d*)|(^\.\d+)$/ ) { return 0 ;}
  return 1;
}

sub set_status { # return worst status with this order : OK, unknwonw, warning, critical 
  my $new_status=shift;
  my $cur_status=shift;
  if (($cur_status == 0)|| ($new_status==$cur_status)){ return $new_status; }
  if ($new_status==3) { return $cur_status; }
  if ($new_status > $cur_status) {return $new_status;}
  return $cur_status;
}

sub help {
   print "\nSNMP UPS Monitor for Nagios version ",$Version,"\n";
   print "GPL Licence, (c)2009 Jrgen Vigna\n\n";
   print_usage();
   print <<EOT;
-v, --verbose
   print extra debugging information 
-h, --help
   print this help message
-H, --hostname=HOST
   name or IP address of host to check
-C, --community=COMMUNITY NAME
   community name for the host's SNMP agent (implies v1 protocol)
-2, --v2c
   Use snmp v2c
-l, --login=LOGIN ; -x, --passwd=PASSWD
   Login and auth password for snmpv3 authentication 
   If no priv password exists, implies AuthNoPriv 
-X, --privpass=PASSWD
   Priv password for snmpv3 (AuthPriv protocol)
-L, --protocols=<authproto>,<privproto>
   <authproto> : Authentication protocol (md5|sha : default md5)
   <privproto> : Priv protocole (des|aes : default des) 
-P, --port=PORT
   SNMP port (Default 161)
-T, --type=general|netvision|socomec|apcats
	Environemental check : 
		general   : general UPS status
		netvision : voltage,battery,load and alerts
		socomec   : voltage,battery,load and alerts
		generex   : battery,status,output and alerts
		chloride  : battery,status,output and alerts (same as generex)
		apcups    : battery,input voltage,output load
		apcats    : input source, load
		mge       : battery,status,input voltage,output voltage and alerts
		nvenergy  : netvision energy stations battery,input voltage,output
 		delta     : status,output load (from generex)
-B, --battery=<%battery left>
   Warning,Critical minimum battery level in percent (default: 80,50)
-O, --load=<max load>
   Warning,Critical Maximum Load of UPS before giving a warning (default: 75,90)
-A, --voltage=<max voltage>
   Warning Minimum Voltage of UPS before giving a warning (default: 200)
-o, --output_lines=<lines>
   Number of output lines to check (default: 1)
-i, --input_lines=<lines>
   Number of input lines to check (default: 1)
-f, --perfparse
   Perfparse compatible output
-t, --timeout=INTEGER
   timeout for SNMP in seconds (Default: 5)
-V, --version
   prints version number
EOT
}

# For verbose output
sub verb { my $t=shift; print $t,"\n" if defined($o_verb) ; }

sub check_options {
    Getopt::Long::Configure ("bundling");
    GetOptions(
   	'v'	=> \$o_verb,		'verbose'	=> \$o_verb,
        'h'     => \$o_help,    	'help'        	=> \$o_help,
        'H:s'   => \$o_host,		'hostname:s'	=> \$o_host,
        'p:i'   => \$o_port,   		'port:i'	=> \$o_port,
        'C:s'   => \$o_community,	'community:s'	=> \$o_community,
	'l:s'	=> \$o_login,		'login:s'	=> \$o_login,
	'x:s'	=> \$o_passwd,		'passwd:s'	=> \$o_passwd,
	'X:s'	=> \$o_privpass,	'privpass:s'	=> \$o_privpass,
	'L:s'	=> \$v3protocols,	'protocols:s'	=> \$v3protocols,   
        't:i'   => \$o_timeout,       	'timeout:i'     => \$o_timeout,
	'V'	=> \$o_version,		'version'	=> \$o_version,

	'2'     => \$o_version2,        'v2c'           => \$o_version2,
        'f'     => \$o_perf,            'perfparse'     => \$o_perf,
	'T:s'	=> \$o_check_type,	'type:s'	=> \$o_check_type,
        'B:s'   => \$o_battery,         'battery:s'    	=> \$o_battery,
        'O:s'   => \$o_load,            'load:s'        => \$o_load,
        'A:i'   => \$o_voltage,         'voltage:i'     => \$o_voltage,
        'o:i'   => \$o_output_lines,    'output_lines:i' => \$o_output_lines,
        'i:i'   => \$o_input_lines,     'input_lines:i' => \$o_input_lines
	);
    # check the -T option
    my $T_option_valid=0; 
    foreach (@valid_types) { if ($_ eq $o_check_type) {$T_option_valid=1} };
    if ( $T_option_valid == 0 ) 
    {print "Invalid check type (-T)!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
    # Basic checks
    if (defined($o_timeout) && (isnnum($o_timeout) || ($o_timeout < 2) || ($o_timeout > 60))) 
        { print "Timeout must be >1 and <60 !\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
    if (!defined($o_timeout)) {$o_timeout=5;}
    if (defined ($o_help) ) { help(); exit $ERRORS{"UNKNOWN"}};
    if (defined($o_version)) { p_version(); exit $ERRORS{"UNKNOWN"}};
    if ( ! defined($o_host) ) # check host and filter 
	{ print_usage(); exit $ERRORS{"UNKNOWN"}}
    # check snmp information
    if ( !defined($o_community) && (!defined($o_login) || !defined($o_passwd)) )
	  { print "Put snmp login info!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
    if ((defined($o_login) || defined($o_passwd)) && (defined($o_community) || defined($o_version2)) )
	  { print "Can't mix snmp v1,2c,3 protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
    if (defined ($v3protocols)) {
        if (!defined($o_login)) { print "Put snmp V3 login info with protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
        my @v3proto=split(/,/,$v3protocols);
        if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
        if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
        if ((defined ($v3proto[1])) && (!defined($o_privpass))) {
            print "Put snmp V3 priv login info with priv protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
    }
    if (defined($o_battery)) {
        my @temp_array=split(/,/,$o_battery);
        if ($#temp_array != 1) {
            print "Battery Warning,Critical options wrong format use w,c \n";
            print_usage();
            exit $ERRORS{"UNKNOWN"};
        }
        $o_w_battery=$temp_array[0];
        $o_c_battery=$temp_array[1];
        if ($o_w_battery < $o_c_battery) {
            print "Battery Capacity Warning must be > Critical level\n";
            print_usage();
            exit $ERRORS{"UNKNOWN"};
        }
    }
    if (defined($o_load)) {
        my @temp_array=split(/,/,$o_load);
        verb("$o_load,$#temp_array ");
        if ($#temp_array != 1) {
            print "Load Warning,Critical options wrong format use w,c \n";
            print_usage();
            exit $ERRORS{"UNKNOWN"};
        }
        $o_w_load=$temp_array[0];
        $o_c_load=$temp_array[1];
        verb("-> $o_w_load,$o_c_load");
        if ($o_w_load > $o_c_load) {
            print "Load Warning must be < Critical level\n";
            print_usage();
            exit $ERRORS{"UNKNOWN"};
        }
    }
}

########## MAIN #######

check_options();

# Check gobal timeout if snmp screws up
if (defined($TIMEOUT)) {
  verb("Alarm at $TIMEOUT + 5");
  alarm($TIMEOUT+5);
} else {
  verb("no global timeout defined : $o_timeout + 10");
  alarm ($o_timeout+10);
}

$SIG{'ALRM'} = sub {
 print "No answer from host\n";
 exit $ERRORS{"UNKNOWN"};
};

# Connect to host
my ($session,$error);
if ( defined($o_login) && defined($o_passwd)) {
  # SNMPv3 login
  verb("SNMPv3 login");
    if (!defined ($o_privpass)) {
  verb("SNMPv3 AuthNoPriv login : $o_login, $o_authproto");
    ($session, $error) = Net::SNMP->session(
      -hostname   	=> $o_host,
      -version		=> '3',
      -username		=> $o_login,
      -authpassword	=> $o_passwd,
      -authprotocol	=> $o_authproto,
      -timeout          => $o_timeout
    );  
  } else {
    verb("SNMPv3 AuthPriv login : $o_login, $o_authproto, $o_privproto");
    ($session, $error) = Net::SNMP->session(
      -hostname   	=> $o_host,
      -version		=> '3',
      -username		=> $o_login,
      -authpassword	=> $o_passwd,
      -authprotocol	=> $o_authproto,
      -privpassword	=> $o_privpass,
	  -privprotocol => $o_privproto,
      -timeout          => $o_timeout
    );
  }
} else {
	if (defined ($o_version2)) {
		# SNMPv2 Login
		verb("SNMP v2c login");
		  ($session, $error) = Net::SNMP->session(
		 -hostname  => $o_host,
		 -version   => 2,
		 -community => $o_community,
		 -port      => $o_port,
		 -timeout   => $o_timeout
		);
  	} else {
	  # SNMPV1 login
	  verb("SNMP v1 login");
	  ($session, $error) = Net::SNMP->session(
		-hostname  => $o_host,
		-community => $o_community,
		-port      => $o_port,
		-timeout   => $o_timeout
	  );
	}
}
if (!defined($session)) {
   printf("ERROR opening session: %s.\n", $error);
   exit $ERRORS{"UNKNOWN"};
}

my $exit_val=undef;
my $perfdata = undef;
my $general_perfdata = undef;

if ($o_check_type eq "socomec") {
	$o_check_type="netvision";
}
if ($o_check_type eq "chloride") {
	$o_check_type="generex";
}

#
### Now check general UPS statuses if available
#
my $general_exist=0;
my $global_status=0;
my $output="";
my $general_output="";

verb("Checking General UPS Status");
my @snmpoids;
push (@snmpoids,$general_battery_status);
push (@snmpoids,$general_output_source);

my $result = $session->get_request(-varbindlist => \@snmpoids);

if (defined($result)) {
	$general_exist=1;
	$general_battery_status = $result->{$general_battery_status};
	$general_output_source = $result->{$general_output_source};
	if (($general_output_source !~ /^noSuch/) && ($general_battery_status !~ /^noSuch/)) {
	    $general_output = "UPS State is ".$upsOutputSources{$general_output_source}.". Battery State is ".$upsBatteryStatuses{$general_battery_status}.".";

	    if ($general_output_source == 3) { # normal
	        $global_status = 0;
	    } elsif (($general_output_source == 6) || # booster
	        ($general_output_source == 7)) { # reducer
	        $global_status = 1;
	    } else {
	        $global_status = 2;
	    }

	    if (($global_status eq 0) && ($general_battery_status != 2)) { # battery low or depleted
	        $global_status = 1;
	    }
	}
} else {
    verb("General Status OID $general_battery_status and output source $general_output_source not found\n");
}

if ($o_check_type eq "general") {
verb("Checking General UPS Values");
my $resultat;
my $output_exist=0;
my $input_exists=0;
my $capacity_exist=0;
my $status=0;

# Get Battery table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_request($general_capacity)
                : $session->get_request(-varbindlist => [$general_capacity]);

if (defined($resultat)) {
    $capacity_exist=1;
    my $capacity = $resultat->{$general_capacity};
    verb("OID : $general_capacity, Desc : $capacity");
    if ($capacity < $o_c_battery) {
        $status = 2;
        $output .= " Battery Level ($capacity)"
    } elsif ($capacity < $o_w_battery) {
        $status = 1;
        $output .= " Battery Level ($capacity)"
    }
    $perfdata .= " capacity=$capacity;$o_w_battery;$o_c_battery;0;100";
} else {
    verb("Capacity not found!")
}
$global_status=set_status($status,$global_status);

$status=0;
# Get Output Load per line
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($general_output_load_table)
                : $session->get_table(Baseoid => $general_output_load_table);

if (defined($resultat)) {
    $output_exist=1;
    my $i=1;
    foreach my $key ( sort keys %$resultat) {
	if ($o_output_lines < $i) { next; }
        my $load = $$resultat{$key};
        verb("OID : $key, Desc : $load : $i");
        if ($load < 0) { next; }
        if ($load > $o_c_load) {
            $output .= " Load$i critical($load)";
            $status=2;
        } elsif ($load > $o_w_load) {
            $output .= " Load$i warning($load)";
            $status=1;
        }
        $perfdata .= " load$i=$load;$o_w_load;$o_c_load;0;100";
        $i++;
    }
} else {
	verb("OID: $general_output_load_table not found");
}
$global_status=set_status($status,$global_status);

$status=0;
# Get Output Voltage per line
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($general_output_voltage_table)
                : $session->get_table(Baseoid => $general_output_voltage_table);

if (defined($resultat)) {
    $output_exist=1;
    my $i=1;
    foreach my $key ( sort keys %$resultat) {
	if ($o_output_lines < $i) { next; }
        my $voltage = $$resultat{$key};
	if ($voltage > 1000) {
		$voltage = $voltage / 10;
	}
        verb("OID : $key, Desc : $voltage : $i");
        if ($voltage < 0) { next; }
        if ($voltage < $o_voltage) {
            $output .= " Output Voltage$i warning($voltage)";
            $status=1;
        }
        $perfdata .= " output_voltage$i=$voltage;$o_voltage;0;0;380";
        $i++;
    }
} else {
	verb("OID: $general_output_voltage_table not found");
}
$global_status=set_status($status,$global_status);

$status=0;
# Get Input Voltage per line
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($general_input_voltage_table)
                : $session->get_table(Baseoid => $general_input_voltage_table);

if (defined($resultat)) {
    $input_exists=1;
    my $i=1;
    foreach my $key ( sort keys %$resultat) {
	if ($o_input_lines < $i) { next; }
        my $voltage = $$resultat{$key};
	if ($voltage > 1000) {
		$voltage = $voltage / 10;
	}
        verb("OID : $key, Desc : $voltage : $i");
        if ($voltage < 0) { next; }
        if ($voltage < $o_voltage) {
            $output .= " Input Voltage$i warning($voltage)";
            $status=1;
        }
        $perfdata .= " input_voltage$i=$voltage;$o_voltage;0;0;380";
        $i++;
    }
} else {
	verb("OID: $general_input_voltage_table not found");
}
$global_status=set_status($status,$global_status);

if (defined($general_output)) {
    $output = $general_output . $output;
}

if ($o_perf && defined($perfdata)) {
    $output .= "|$perfdata";
}

if ($global_status==0) {
  print "OK - All seems fine $output\n";
  exit $ERRORS{"OK"};
}

if ($global_status==1) {
  print "WARNING - $output\n";
  exit $ERRORS{"WARNING"};
}

if ($global_status==2) {
  print "CRITICAL - $output\n";
  exit $ERRORS{"CRITICAL"};
}

print "UNKNOWN - shouldn't be here\n";
exit $ERRORS{"UNKNOWN"};
}

############# Net Vision checks
if ($o_check_type eq "netvision") {

verb("Checking netvision ups");

my $resultat;
# status : 0=ok, 1=warning, 2=critial
my ($alerts_status,$capacity_status,$load_status,$voltage_status)=(0,0,0,0);
my ($alerts_exist,$capacity_exist,$load_exist,$voltage_exist)=(0,0,0,0);

# Get Alerts table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($netvision_alert_table)
                : $session->get_table(Baseoid => $netvision_alert_table);

if (defined($resultat)) {
    $alerts_exist=1;
    my $i=0;
    foreach my $key ( keys %$resultat) {
        verb("OID : $key, Desc : $$resultat{$key} : $i");
        if ($$resultat{$key} != 0) {
            if ($alerts_status == 0) {
                $output = "Alerts aktiv:"
            }
            $alerts_status=1;
            $output .= " $netvision_alert_leaf[$i]($$resultat{$key})"
        }
        $i++;
    }
}

$global_status=set_status($alerts_status,$global_status);

# Get Battery table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_request($netvision_capacity)
                : $session->get_request(-varbindlist => [$netvision_capacity]);

if (defined($resultat)) {
    $capacity_exist=1;
    my $capacity = $resultat->{$netvision_capacity};
    verb("OID : $netvision_capacity, Desc : $capacity");
    if ($capacity < $o_c_battery) {
        $capacity_status = 2;
        $output .= " Battery Level ($capacity)"
    } elsif ($capacity < $o_w_battery) {
        $capacity_status = 1;
        $output .= " Battery Level ($capacity)"
    }
    $perfdata .= " capacity=$capacity;$o_w_battery;$o_c_battery;0;100";
} else {
    verb("Capacity not found!")
}

$global_status=set_status($capacity_status,$global_status);

# Get Load table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($netvision_load_table)
                : $session->get_table(Baseoid => $netvision_load_table);

if (defined($resultat)) {
    $load_exist=1;
    my $i=1;
    foreach my $key ( keys %$resultat) {
        my $load = $$resultat{$key};
        if ($load < 0) { next; }
        verb("OID : $key, Desc : $load : $i");
        if ($load > $o_c_load) {
            $output .= " Load$i critical($load)";
            $load_status=2;
        } elsif ($load > $o_w_load) {
            $output .= " Load$i warning($load)";
            $load_status=1;
        }
        $perfdata .= " load$i=$load;$o_w_load;$o_c_load;0;100";
        $i++;
    }
}

$global_status=set_status($load_status,$global_status);

# Get Voltage table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($netvision_voltage_table)
                : $session->get_table(Baseoid => $netvision_voltage_table);

if (defined($resultat)) {
    $voltage_exist=1;
    my $i=1;
    foreach my $key ( keys %$resultat) {
        my $voltage = $$resultat{$key} / 10;
        if ($voltage < 0) { next; }
        verb("OID : $key, Desc : $voltage : $i");
        if ($voltage < $o_voltage) {
            $output .= " Voltage$i warning($voltage)";
            $voltage_status=1;
        }
        $perfdata .= " voltage$i=$voltage;$o_voltage;0;0;380";
        $i++;
    }
}

$global_status=set_status($voltage_status,$global_status);

$session->close;

verb ("status : $global_status");

if ( ($alerts_exist+$capacity_exist+$load_exist+$voltage_exist) == 0) {
  if ($general_exist) {
    $output .= " No special Netvision Data found";
  } else {
    print "No UPS informations found : UNKNOWN\n";
    exit $ERRORS{"UNKNOWN"};
  }
}

if (defined($general_output)) {
    $output = $general_output . $output;
}

if ($o_perf && defined($perfdata)) {
    $output .= "|$perfdata";
}

if ($global_status==0) {
  print "OK - All seems fine $output\n";
  exit $ERRORS{"OK"};
}

if ($global_status==1) {
  print "WARNING - $output\n";
  exit $ERRORS{"WARNING"};
}

if ($global_status==2) {
  print "CRITICAL - $output\n";
  exit $ERRORS{"CRITICAL"};
}
}

#
############# Generex General UPS
#
if ($o_check_type eq "generex") {

verb("Checking Generex General UPS (chloride)");

my $resultat;
# status : 0=ok, 1=warning, 2=critial
my ($alerts_status,$battery_status,$output_status,$input_status,$load_status)=(0,0,0,0,0);
my ($alerts_exist,$battery_exist,$output_exist,$input_exist,$load_exist)=(0,0,0,0,0);
my $global_status=0;

# Get Alerts table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_request($generex_alarms)
                : $session->get_request(-varbindlist => [$generex_alarms]);

if (defined($resultat)) {
    $alerts_exist=1;
    my $i=0;
    my $alarms;
    foreach my $key ( keys %$resultat) {
        verb("OID : $key, Desc : $$resultat{$key} : $i");
	$alarms = $$resultat{$key};
        if ($alarms > 0) {
            if ($alarms > 1) {
                $output .= ", $alarms alarms present";
            } else {
                $output .= ", 1 alarm present";
            }
            $alerts_status=2;
        }
        $i++;
    }
}

$global_status=$alerts_status;

# Get Battery table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($generex_battery_base)
                : $session->get_table(Baseoid => $generex_battery_base);

if (defined($resultat)) {
    $battery_exist=1;
    my $i=0;
    my $value;
    foreach my $key ( keys %$resultat) {
        verb("OID : $key, Desc : $$resultat{$key} : $i");
        $value = $$resultat{$key};
	if ( $key =~ /$generex_battery_status/ ) {
            if ($value == 1) {
                $output .= "State unknown. ";
                $battery_status=set_status(3,$battery_status);
            } elsif ($value == 2) {
                $output .= "State batteryNormal. ";
            } elsif ($value == 3) {
                $output .= "State batteryLow. ";
                $battery_status=set_status(1,$battery_status);
            } elsif ($value == 4) {
                $output .= "State batteryDepleted. ";
                $battery_status=2;
            } else {
                $output .= "State invalid. ";
                $battery_status=set_status(3,$battery_status);
            }
       } elsif ( $key =~ /$generex_battery_capacity/ ) {
           if ($value < $o_c_battery) {
               $battery_status = 2;
               $output .= "Battery Level ($value%). ";
           } elsif ($value < $o_w_battery) {
               $output .= "Battery Level ($value%). ";
               $battery_status=set_status(1,$battery_status);
           }
           $perfdata .= " capacity=$value;$o_w_battery;$o_c_battery;0;100";
       } elsif ( $key =~ /$generex_battery_mintot/ ) {
           $output .= "Battery Minutes remaining $value. ";
           $perfdata .= " battery_min_remaining=$value";
       }
    }
} else {
    verb("Battery not found!");
}

$global_status=set_status($battery_status,$global_status);

# Get Load table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($generex_output_voltage)
                : $session->get_table(Baseoid => $generex_output_voltage);

if (defined($resultat)) {
    $output_exist=1;
    my $i=1;
    foreach my $key ( keys %$resultat) {
        my $voltage = $$resultat{$key};
        if ($voltage < 0) { next; }
        verb("OID : $key, Desc : $voltage : $i");
        if ($voltage < $o_voltage) {
            $output .= ", Voltage$i warning($voltage)";
            $output_status=1;
        }
        $perfdata .= " outvol$i=$voltage;$o_voltage;0;0;380";
        $i++;
    }
}

$global_status=set_status($output_status,$global_status);

# Get Output OverLoad per line
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($generex_output_load)
                : $session->get_table(Baseoid => $generex_output_load);

if (defined($resultat)) {
    $load_exist=1;
    my $i=0;
    my $alarms;
    foreach my $key ( keys %$resultat) {
        verb("OID : $key, Desc : $$resultat{$key} : $i");
	my $load = $$resultat{$key};
        if ($load > $o_c_load) {
            $output .= " Load$i critical($load)";
            $load_status=2;
        } elsif ($load > $o_w_load) {
            $output .= " Load$i warning($load)";
            $load_status=1;
        }
	$output .= " Load $load%.";
        $perfdata .= " load=$load;$o_w_load;$o_c_load;0;100";
        $i++;
    }
} else {
	verb("OID: $generex_output_load not found");
}

$global_status=set_status($load_status,$global_status);
$session->close;

verb ("status : $global_status");

if ( ($general_exist+$alerts_exist+$battery_exist+$load_exist) == 0) {
  print "No UPS informations found : UNKNOWN\n";
  exit $ERRORS{"UNKNOWN"};
}

if (defined($general_output) && !defined($output)) {
    $output = $general_output;
}

if ($o_perf && defined($perfdata)) {
    $output .= "|$perfdata";
}

if ($global_status==0) {
  print "OK - All seems fine $output\n";
  exit $ERRORS{"OK"};
}

if ($global_status==1) {
  print "WARNING - $output\n";
  exit $ERRORS{"WARNING"};
}

if ($global_status==2) {
  print "CRITICAL - $output\n";
  exit $ERRORS{"CRITICAL"};
}
}

############# APC UPS checks

if ($o_check_type eq "apcups") {

verb("Checking APC ups");

my $resultat;
# status : 0=ok, 1=warning, 2=critial
my ($alerts_status,$capacity_status,$load_status,$voltage_status)=(0,0,0,0);
my ($alerts_exist,$capacity_exist,$load_exist,$voltage_exist)=(0,0,0,0);

# Get Battery table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_request($apcups_battery_capacity)
                : $session->get_request(-varbindlist => [$apcups_battery_capacity]);

if (defined($resultat)) {
    $capacity_exist=1;
    my $capacity = $resultat->{$apcups_battery_capacity};
    verb("OID : $apcups_battery_capacity, Desc : $capacity");
    if ($capacity < $o_c_battery) {
        $output .= " CRITICAL Battery Level: $capacity %";
        $capacity_status = 2;
    } elsif ($capacity < $o_w_battery) {
       $output .= " WARNING Battery Level: $capacity %";
        $capacity_status = 1;
    }else{
       $output .= " Batt. Level: $capacity %";
    }
    $perfdata .= " capacity=$capacity;$o_w_battery;$o_c_battery;0;100";
} else {
    verb("Capacity not found!")
}

$global_status=set_status($capacity_status,$global_status);

# Battery temperature
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_request($apcups_battery_temp)
                : $session->get_request(-varbindlist => [$apcups_battery_temp]);

if (defined($resultat)) {
    $capacity_exist=1;
    my $capacity = $resultat->{$apcups_battery_temp};
    verb("OID : $apcups_battery_temp, Desc : $capacity");
    if ($capacity > $o_c_battery_temp) {
        $output .= " CRITCAL Battery Temperature: $capacity";
        $capacity_status = 2;
    } elsif ($capacity > $o_w_battery_temp) {
    $output .= "WARNING Battery Temperature: $capacity";
        $capacity_status = 1;
    }
    $perfdata .= " batt_temp=$capacity;$o_w_battery_temp;$o_c_battery_temp;0;100";
} else {
    verb("Battery Temp not found!")
}

$global_status=set_status($capacity_status,$global_status);

# Battery Voltage
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_request($apcups_battery_voltage)
                : $session->get_request(-varbindlist => [$apcups_battery_voltage]);

if (defined($resultat)) {
    $capacity_exist=1;
    my $capacity = $resultat->{$apcups_battery_voltage};
    verb("OID : $apcups_battery_voltage, Desc : $capacity");
    if ($capacity < $o_voltage) {
        $output .= " CRITICAL Battery Voltage: $capacity";
        $capacity_status = 2;
    }else {
        $output .= " Batt.: $capacity V";
    }
    $perfdata .= " batt_volt=$capacity;0;$o_voltage;;";
} else {
    verb("Battery Voltage not found!")
}
$global_status=set_status($capacity_status,$global_status);

# Get Load table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($apcups_output_load)
                : $session->get_table(Baseoid => $apcups_output_load);

if (defined($resultat)) {
    $load_exist=1;
    my $i=1;
    foreach my $key ( keys %$resultat) {
        my $load = $$resultat{$key};
        if ($load < 0) { next; }
        verb("OID : $key, Desc : $load : $i");
        if ($load > $o_c_load) {
            $output .= " Load$i critical($load)";
            $load_status=2;
        } elsif ($load > $o_w_load) {
            $output .= " Load$i warning($load)";
            $load_status=1;
        } else {
           $output .= " Load$i: $load %";
        }
        $perfdata .= " load$i=$load;$o_w_load;$o_c_load;0;100";
        $i++;
    }
}

$global_status=set_status($load_status,$global_status);

# Get Voltage table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($apcups_input_voltage)
                : $session->get_table(Baseoid => $apcups_input_voltage);

if (defined($resultat)) {
    $voltage_exist=1;
    my $i=1;
    foreach my $key ( keys %$resultat) {
        my $voltage = $$resultat{$key};
        if ($voltage < 0) { next; }
        verb("OID : $key, Desc : $voltage : $i");
        if ($voltage < $o_voltage) {
            $output .= " Input Voltage$i warning($voltage)";
            $voltage_status=1;
        }else {
            $output .= " Input$i $voltage V";
        }
        $perfdata .= " in_voltage$i=$voltage;$o_voltage;0;0;380";
        $i++;
    }
}

$global_status=set_status($voltage_status,$global_status);


$session->close;

verb ("status : $global_status");

if ( ($alerts_exist+$capacity_exist+$load_exist+$voltage_exist) == 0) {
  if ($general_exist) {
    $output .= " No special APC Data found";
  } else {
    print "No  APCUPS informations found : UNKNOWN\n";
    exit $ERRORS{"UNKNOWN"};
  }
}

if (defined($general_output)) {
    $output = $general_output . $output;
}

if ($o_perf && defined($perfdata)) {
    $output .= "|$perfdata";
}

if ($global_status==0) {
  print "OK - All seems fine $output\n";
  exit $ERRORS{"OK"};
}

if ($global_status==1) {
  print "WARNING - $output\n";
  exit $ERRORS{"WARNING"};
}

if ($global_status==2) {
  print "CRITICAL - $output\n";
  exit $ERRORS{"CRITICAL"};
}
}


############# APC ATS checks
if ($o_check_type eq "apcats") {

verb("Checking APC ATS");

my $resultat;
# status : 0=ok, 1=warning, 2=critial
my ($input_status,$capacity_status,$load_status,$voltage_status)=(0,0,0,0);
my ($input_exist,$capacity_exist,$load_exist,$voltage_exist)=(0,0,0,0);

# Get Alerts table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($apcats_input_table)
                : $session->get_table(Baseoid => $apcats_input_table);

if (defined($resultat)) {
    $input_exist=1;
    my $i=0;
    my $temp;
    my $voltage;
    my $perfa;
    my $perfb;
    foreach my $key ( keys %$resultat) {
        verb("OID : $key, Desc : $$resultat{$key} : $i");
        $voltage=$$resultat{$key};
        if ($key =~ $apcats_input_a) {
            $temp="A";
            $perfa=" inputa=$voltage;$o_voltage;0;0;390";
        } elsif ($key =~ $apcats_input_b) {
            $temp="B";
            $perfb=" inputb=$voltage;$o_voltage;0;0;390";
        }
        if ($$resultat{$key} == 0) {
            $output .= " Input-$temp not connected";
            $input_status=2;
        } elsif ($voltage < $o_voltage) {
            $output .= " Input-$temp Voltage warning($voltage<$o_voltage)";
            if ($input_status < 2) {
                $input_status=1;
            }
        }
        $i++;
    }
    $perfdata.="$perfa $perfb";
}

$global_status=set_status($input_status,$global_status);

$session->close;

verb ("status : $global_status");

if ( ($input_exist+$capacity_exist+$load_exist+$voltage_exist) == 0) {
  if ($general_exist) {
    $output .= " No special APC Data found";
  } else {
    print "No UPS informations found : UNKNOWN\n";
    exit $ERRORS{"UNKNOWN"};
  }
}

if (defined($general_output)) {
    $output = $general_output . $output;
}

if ($o_perf && defined($perfdata)) {
    $output .= "|$perfdata";
}

if ($global_status==0) {
  print "OK - All seems fine $output\n";
  exit $ERRORS{"OK"};
}

if ($global_status==1) {
  print "WARNING - $output\n";
  exit $ERRORS{"WARNING"};
}

if ($global_status==2) {
  print "CRITICAL - $output\n";
  exit $ERRORS{"CRITICAL"};
}
}

#
############# MGE UPS
#
if ($o_check_type eq "mge") {
verb("Checking MGE UPS Values");
my $resultat;
my $status_exist=0;
my $output_exist=0;
my $input_exists=0;
my $capacity_exist=0;
my $status=0;

verb("Checking General MGE UPS Status");
#my $mge_battery_fault      = ".1.3.6.1.4.1.705.1.5.9.0"; # 1 yes, 2 no
#my $mge_input_outage       = ".1.3.6.1.4.1.705.1.6.4.0"; # 1: ok, 2: voltage out tolerance, 3: freq out tolerance, 4: no voltage.
my @snmpoids;
push (@snmpoids,$mge_battery_fault);
push (@snmpoids,$mge_input_outage);

$resultat = $session->get_request(-varbindlist => \@snmpoids);

if (defined($resultat)) {
	$status_exist=1;
	my $battery_status = $resultat->{$mge_battery_fault};
	my $input_status = $resultat->{$mge_input_outage};
        verb("BS: $battery_status\nIS: $input_status\n");
	if ($battery_status !~ /^noSuch/) {
            if ($battery_status == 1) {
	      $output = "UPS Battery State is FAULTY";
	      $global_status = 2;
            } else {
              $output = "UPS Battery State is OK";
            }
	}
	if ($input_status !~ /^noSuch/) {
            if ($input_status == 2) {
	      $output .= ", Input Voltage out of tolerance";
	      $global_status = 2;
            } elsif ($input_status == 3) {
	      $output .= ", Input Freq out of tolerance";
	      $global_status = 2;
            } elsif ($input_status == 4) {
	      $output .= ", No input Voltage";
	      $global_status = 2;
            }
	}
} else {
    verb("MGE General Status OID $mge_battery_fault and input status $mge_input_outage not found\n");
}

# Get Battery percentage
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_request($mge_battery_percentage)
                : $session->get_request(-varbindlist => [$mge_battery_percentage]);

if (defined($resultat)) {
    $capacity_exist=1;
    my $capacity = $resultat->{$mge_battery_percentage};
    verb("OID : $mge_battery_percentage, Desc : $capacity");
    if ($capacity < $o_c_battery) {
        $status = 2;
        $output .= " Battery Level ($capacity)"
    } elsif ($capacity < $o_w_battery) {
        $status = 1;
        $output .= " Battery Level ($capacity)"
    }
    $perfdata .= " capacity=$capacity;$o_w_battery;$o_c_battery;0;100";
} else {
    verb("Capacity not found!")
}
$global_status=set_status($status,$global_status);

$status=0;
# Get Output Load per line
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($general_output_load_table)
                : $session->get_table(Baseoid => $general_output_load_table);

if (defined($resultat)) {
    $output_exist=1;
    my $i=1;
    foreach my $key ( sort keys %$resultat) {
	if ($o_output_lines < $i) { next; }
        my $load = $$resultat{$key};
        verb("OID : $key, Desc : $load : $i");
        if ($load < 0) { next; }
        if ($load > $o_c_load) {
            $output .= " Load$i critical($load)";
            $status=2;
        } elsif ($load > $o_w_load) {
            $output .= " Load$i warning($load)";
            $status=1;
        }
        $perfdata .= " load$i=$load;$o_w_load;$o_c_load;0;100";
        $i++;
    }
} else {
	verb("OID: $general_output_load_table not found");
}
$global_status=set_status($status,$global_status);

$status=0;
# Get Output Voltage per line
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($general_output_voltage_table)
                : $session->get_table(Baseoid => $general_output_voltage_table);

if (defined($resultat)) {
    $output_exist=1;
    my $i=1;
    foreach my $key ( sort keys %$resultat) {
	if ($o_output_lines < $i) { next; }
        my $voltage = $$resultat{$key};
	if ($voltage > 1000) {
		$voltage = $voltage / 10;
	}
        verb("OID : $key, Desc : $voltage : $i");
        if ($voltage < 0) { next; }
        if ($voltage < $o_voltage) {
            $output .= " Output Voltage$i warning($voltage)";
            $status=1;
        }
        $perfdata .= " output_voltage$i=$voltage;$o_voltage;0;0;380";
        $i++;
    }
} else {
	verb("OID: $general_output_voltage_table not found");
}
$global_status=set_status($status,$global_status);

$status=0;
# Get Input Voltage per line
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($general_input_voltage_table)
                : $session->get_table(Baseoid => $general_input_voltage_table);

if (defined($resultat)) {
    $input_exists=1;
    my $i=1;
    foreach my $key ( sort keys %$resultat) {
	if ($o_input_lines < $i) { next; }
        my $voltage = $$resultat{$key};
	if ($voltage > 1000) {
		$voltage = $voltage / 10;
	}
        verb("OID : $key, Desc : $voltage : $i");
        if ($voltage < 0) { next; }
        if ($voltage < $o_voltage) {
            $output .= " Input Voltage$i warning($voltage)";
            $status=1;
        }
        $perfdata .= " input_voltage$i=$voltage;$o_voltage;0;0;380";
        $i++;
    }
} else {
	verb("OID: $general_input_voltage_table not found");
}
$global_status=set_status($status,$global_status);

if (defined($general_output)) {
    $output = $general_output . $output;
}

if ($o_perf && defined($perfdata)) {
    $output .= "|$perfdata";
}

if ($global_status==0) {
  print "OK - $output\n";
  exit $ERRORS{"OK"};
}

if ($global_status==1) {
  print "WARNING - $output\n";
  exit $ERRORS{"WARNING"};
}

if ($global_status==2) {
  print "CRITICAL - $output\n";
  exit $ERRORS{"CRITICAL"};
}

print "UNKNOWN - shouldn't be here\n";
exit $ERRORS{"UNKNOWN"};
}


############# Net Vision Energy Station checks
if ($o_check_type eq "nvenergy") {

verb("Checking netvision energy station ups");

my $resultat;
# status : 0=ok, 1=warning, 2=critial
my ($battery_status,$capacity_status,$inpvol_status,$output_status,$outvol_status)=(0,0,0,0,0);
my ($battery_exist,$capacity_exist,$inpvol_exist,$output_exist,$outvol_exist)=(0,0,0,0,0);

# Get Battery table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_request($netvision_energy_batstatus)
                : $session->get_request(-varbindlist => [$netvision_energy_batstatus]);

if (defined($resultat)) {
    $battery_exist=1;
    my $batstatus = $resultat->{$netvision_energy_batstatus};
    verb("OID : $netvision_energy_batstatus, Desc : $batstatus");
    if ($batstatus < 2 || $batstatus > 5) {
        $battery_status = 3;
        $output .= " Battery Status (unknown)"
    } elsif ($batstatus == 3) {
        $battery_status = 1;
        $output .= " Battery Status (batteryDischarging)"
    } elsif ($batstatus == 4) {
        $battery_status = 2;
        $output .= " Battery Status (batteryDischarged)"
    } elsif ($batstatus == 5) {
        $battery_status = 2;
        $output .= " Battery Status (batteryFailure)"
    } else {
        $output .= " Battery Status (normal)"
    }
} else {
    verb("Battery Status not found!")
}

$global_status=set_status($battery_status,$global_status);

# Get Battery Capacity
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_request($netvision_energy_batcapacity)
                : $session->get_request(-varbindlist => [$netvision_energy_batcapacity]);

if (defined($resultat)) {
    $capacity_exist=1;
    my $capacity = $resultat->{$netvision_energy_batcapacity};
    verb("OID : $netvision_energy_batcapacity, Desc : $capacity");
    if ($capacity < $o_c_battery) {
        $capacity_status = 2;
        $output .= " Battery Level ($capacity)"
    } elsif ($capacity < $o_w_battery) {
        $capacity_status = 1;
        $output .= " Battery Level ($capacity)"
    }
    $perfdata .= " capacity=$capacity;$o_w_battery;$o_c_battery;0;100";
} else {
    verb("Capacity not found!")
}

$global_status=set_status($capacity_status,$global_status);

# Get Input Voltage table
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($netvision_energy_inpvoltage)
                : $session->get_table(Baseoid => $netvision_energy_inpvoltage);

if (defined($resultat)) {
    $inpvol_exist=1;
    my $i=1;
    foreach my $key ( keys %$resultat) {
        my $voltage = $$resultat{$key};
        if ($voltage < 0) { next; }
        verb("OID : $key, Desc : $voltage : $i");
        if ($voltage < $o_voltage) {
            $output .= " Voltage$i warning($voltage)";
            $inpvol_status=1;
        }
        $perfdata .= " input_voltage$i=$voltage;$o_voltage;0;0;380";
        $i++;
    }
}

$global_status=set_status($inpvol_status,$global_status);

# Get Output Status
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_request($netvision_energy_outstatus)
                : $session->get_request(-varbindlist => [$netvision_energy_outstatus]);

if (defined($resultat)) {
    $output_exist=1;
    my $outstatus = $resultat->{$netvision_energy_outstatus};
    verb("OID : $netvision_energy_outstatus, Desc : $outstatus");
    if ($outstatus < 2 || $outstatus > 4) {
        $output_status = 3;
        $output .= " Output Status (other/unknown)"
    } elsif ($outstatus == 2) {
        $output_status = 2;
        $output .= " Output Status (none)"
    } elsif ($outstatus == 4) {
        $output_status = 1;
        $output .= " Output Status (battery)"
    } else {
        $output .= " Output Status (normal)"
    }
} else {
    verb("Output Status not found!")
}

$global_status=set_status($output_status,$global_status);

# Get Output Voltage
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_request($netvision_energy_outvoltage)
                : $session->get_request(-varbindlist => [$netvision_energy_outvoltage]);

if (defined($resultat)) {
    $outvol_exist=1;
    my $voltage = $resultat->{$netvision_energy_outvoltage} * 0.1;
    verb("OID : $netvision_energy_outvoltage, Desc : $voltage");
    $perfdata .= " output_voltage=$voltage";
    $output .= " Output Voltage ($voltage)"
} else {
    verb("Output Voltage not found!")
}

$global_status=set_status($outvol_status,$global_status);

$session->close;

verb ("status : $global_status");

if ( ($battery_exist+$capacity_exist+$inpvol_exist+$output_exist+$outvol_exist) == 0) {
  if ($general_exist) {
    $output .= " No special Netvision Data found";
  } else {
    print "No UPS informations found : UNKNOWN\n";
    exit $ERRORS{"UNKNOWN"};
  }
}

if (defined($general_output)) {
    $output = $general_output . $output;
}

if ($o_perf && defined($perfdata)) {
    $output .= "|$perfdata";
}

if ($global_status==0) {
  print "OK - All seems fine $output\n";
  exit $ERRORS{"OK"};
}

if ($global_status==1) {
  print "WARNING - $output\n";
  exit $ERRORS{"WARNING"};
}

if ($global_status==2) {
  print "CRITICAL - $output\n";
  exit $ERRORS{"CRITICAL"};
}
}

#
############# Delta General UPS
#
if ($o_check_type eq "delta") {

verb("Checking Delta General UPS (generex)");

my $resultat;
# status : 0=ok, 1=warning, 2=critial
my ($alerts_status,$battery_status,$output_status,$input_status,$load_status)=(0,0,0,0,0);
my ($alerts_exist,$battery_exist,$output_exist,$input_exist,$load_exist)=(0,0,0,0,0);
my $global_status=0;

verb("Checking General UPS Status");
my @snmpoids;
push (@snmpoids,$general_output_source);

my $result = $session->get_request(-varbindlist => \@snmpoids);

if (defined($result)) {
        my $val = $result->{$general_output_source};
        verb("General OID output source $general_output_source=$val\n");
        if ($val !~ /^noSuch/) {
            $output = "UPS State is ".$upsOutputSources{$val}.".";

            if ($val == 3) { # normal
                $global_status = 0;
            } elsif (($val == 6) || # booster
                ($val == 7)) { # reducer
                $global_status = 1;
            } else {
                $global_status = 2;
            }
        }
} else {
    verb("General OID output source $general_output_source not found\n");
}

# Get Output OverLoad per line
$resultat = (Net::SNMP->VERSION < 4) ?
                  $session->get_table($delta_output_load)
                : $session->get_table(Baseoid => $delta_output_load);

if (defined($resultat)) {
    $load_exist=1;
    my $i=0;
    my $alarms;
    foreach my $key ( keys %$resultat) {
        verb("OID : $key, Desc : $$resultat{$key} : $i");
        my $load = $$resultat{$key} / 10;
        if ($load > $o_c_load) {
            $output .= " Load$i critical($load)";
            $load_status=2;
        } elsif ($load > $o_w_load) {
            $output .= " Load$i warning($load)";
            $load_status=1;
        }
        $output .= " Load $load%.";
        $perfdata .= " load=$load;$o_w_load;$o_c_load;0;100";
        $i++;
    }
} else {
        verb("OID: $generex_output_load not found");
}

$global_status=set_status($load_status,$global_status);
$session->close;

verb ("status : $global_status");

if ( ($general_exist+$alerts_exist+$battery_exist+$load_exist) == 0) {
  print "No UPS informations found : UNKNOWN\n";
  exit $ERRORS{"UNKNOWN"};
}

if (defined($general_output) && !defined($output)) {
    $output = $general_output;
}

if ($o_perf && defined($perfdata)) {
    $output .= "|$perfdata";
}

if ($global_status==0) {
  print "OK - All seems fine $output\n";
  exit $ERRORS{"OK"};
}

if ($global_status==1) {
  print "WARNING - $output\n";
  exit $ERRORS{"WARNING"};
}

if ($global_status==2) {
  print "CRITICAL - $output\n";
  exit $ERRORS{"CRITICAL"};
}
}


exit (3);
